; random_pattern_tile.scm
; by Rob Antonishen
; http://ffaat.pointclark.net

; Version 0.9 (20110805)

; Changes:

; Description
; fills selection with a multiple patterns weighted.

; License:
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; The GNU Public License is available at
; http://www.gnu.org/copyleft/gpl.html
(define (script-fu-random-pattern-tile img inLayer p0 w0 p1 w1 p2 w2 p3 w3 p4 w4)
(let* (
		(width (car (gimp-image-width img)))
		(height (car (gimp-image-height img)))
		(weight 0)
		(patW (car (gimp-pattern-get-info p0)))
		(patH (cadr (gimp-pattern-get-info p0)))
		(xTiles (trunc (/ width patW))) 
		(yTiles (trunc (/ height patH)))		
		(check 0)
		(newlayer 0)
		(x0 0)
		(y0 0)
		(counterX 0)
		(counterY 0)
		(mode (car (gimp-layer-get-mode inLayer)))
		(savedsel 0)
		)

	(gimp-context-push)
	(gimp-image-undo-group-start img)
	
    ; save selection	
	(when (= (car (gimp-selection-is-empty img)) FALSE)
	  (set! savedsel (car (gimp-selection-save img)))
	  (gimp-selection-none img)
	)
	
	;create a working layer
	(set! newlayer (car (gimp-layer-new-from-drawable inLayer img)))
	(gimp-image-add-layer img newlayer (car (gimp-image-get-layer-position img inLayer)))		
	(gimp-layer-set-mode newlayer NORMAL-MODE)

	;clean weights
	(if (equal? p1 "") (set! w1 0))
	(if (equal? p2 "") (set! w2 0))
	(if (equal? p3 "") (set! w3 0))
	(if (equal? p4 "") (set! w4 0))
    (set! weight (+ w0 w1 w2 w3 w4 1))

	;select and fill from tiles
	(while (<= counterX xTiles)
	(begin
      (while (<= counterY yTiles)
	  (begin
	
		(set! x0 (* counterX patW))
		(set! y0 (* counterY patH))

		(gimp-rect-select img x0 y0 patW patH CHANNEL-OP-REPLACE FALSE 0)
		
		(set! check (rand weight))
        (if (<= check w0)
		  (gimp-context-set-pattern p0)
		  (if (<= check (+ w0 w1))
		    (gimp-context-set-pattern p1)
		    (if (<= check (+ w0 w1 w2))
		      (gimp-context-set-pattern p2)
		      (if (<= check (+ w0 w1 w2 w3))
		        (gimp-context-set-pattern p3)
  	            (gimp-context-set-pattern p4)))))
		 
        (gimp-edit-bucket-fill newlayer PATTERN-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
		(set! counterY (+ counterY 1))
      ))
	  (set! counterY 0)
      (set! counterX (+ counterX 1))
	))

    (gimp-selection-none img)
	
	(when (<> savedsel 0)
	  (gimp-selection-load savedsel)
	  (gimp-image-remove-channel img savedsel)
	  (gimp-selection-invert img)
	  (gimp-edit-clear newlayer)
	  (gimp-selection-invert img)
    )	  
	
	(set! inLayer (car (gimp-image-merge-down img newlayer  EXPAND-AS-NECESSARY)))
	(gimp-layer-set-mode inLayer mode)
	
	(gimp-progress-end)
	(gimp-image-undo-group-end img)
	(gimp-displays-flush)
	(gimp-context-pop)
)
)


(script-fu-register "script-fu-random-pattern-tile"
					"<Image>/Edit/_Fill with Random Pattern Tiles..."
					"Fill the selection with random pattern tiles.  Tile size comes from the first pattern."
					"Rob Antonishen"
					"Rob Antonishen"
					"Aug 2011"
					"RGB* GRAY*"
					SF-IMAGE       "Image"         0
					SF-DRAWABLE    "Drawable"      0
					SF-PATTERN     "Base Pattern"        ""
					SF-ADJUSTMENT  "Base Pattern Weight" (list 1 1 10 1 10 0 SF-SPINNER)
					SF-PATTERN     "Alternate Pattern 1" ""
					SF-ADJUSTMENT  "Alternate Weight 1"  (list 0 0 10 1 10 0 SF-SPINNER)
					SF-PATTERN     "Alternate Pattern 2" ""
					SF-ADJUSTMENT  "Alternate Weight 2"  (list 0 0 10 1 10 0 SF-SPINNER)
					SF-PATTERN     "Alternate Pattern 3" ""
					SF-ADJUSTMENT  "Alternate Weight 3"  (list 0 0 10 1 10 0 SF-SPINNER)
					SF-PATTERN     "Alternate Pattern 4" ""
					SF-ADJUSTMENT  "Alternate Weight 4"  (list 0 0 10 1 10 0 SF-SPINNER)
					)